import { Planets } from '../db/mongodb';
import type User from '../../types/User';
import { ObjectId } from 'mongodb';
import type Planet from '../../types/Planet';
import { getUserById } from './users';
import type Building from '../../types/Building';
import type Ship from '../../types/Ship';
import type DBResource from '../../types/DBResource';

export const getAllPlanets = async (options?: { fetchUserPlanets: boolean }) => {
    const planets = await Planets();
    const fetched = (await planets.find({}).toArray()).map(async planet => {
        planet.owner = await getUserById(planet.owner);
        return planet;
    })

    return Promise.all(fetched) as unknown as Array<Planet>;
}

export const createPlanet = async (options: { name: string, owner?: User, ownerId?: ObjectId, fields?: number }) => {    
    if(options.owner !== undefined && options.ownerId !== undefined) throw new Error("Duplicate identifier");

    let user: ObjectId;
    if(options.ownerId) user = options.ownerId;
    else if(options.owner) user = options.owner._id;
    else throw new Error("Unknown error");

    if(options.fields === undefined) options.fields = 20;
    
    //@ts-ignore TODO: find another way of handling IDs from types before inserting to database
    const planet = {
        owner: user,
        name: options.name,
        fields: options.fields,
        resources: new Array<DBResource>,
        buildings: new Array<Building>,
        ships: new Array<Ship>
    }

    await (await Planets()).insertOne(planet);
}

export const getPlanetById = async (id: ObjectId) => {
    const planets = await Planets();
    const planet = await planets.findOne({
        _id: id
    });

    if(!planet) return null;

    planet.owner = await getUserById(planet.owner);
    return planet as Planet;
}

export const createOrUpgradeBuilding = async (planetId: ObjectId, building: Building) => {
    const planet = await getPlanetById(planetId);
    if(!planet) throw new Error("Planet not found");

    const buildingIndex = planet.buildings.findIndex(b => b.id === building.id);
    if(buildingIndex === -1) {
        planet.buildings.push(building);
    } else {
        planet.buildings[buildingIndex].level++;
    }

    const planets = await Planets();
    await planets.updateOne({
        _id: planetId
    }, {
        $set: {
            buildings: planet.buildings
        }
    })
}