import { ObjectId } from "mongodb"
import { Planets } from "../db/mongodb";
import type DBResource from "../../types/DBResource";
import { getPlanetById } from "../db/planets";

export const createInitialResources = async (planetId: ObjectId) => {
    const resources: Array<DBResource> = [
        {
            name: "coal",
            amount: 11,
            lastUpdated: new Date(),
            perHourMiningRate: 11
        },
        {
            name: "iron",
            amount: 22,
            lastUpdated: new Date(),
            perHourMiningRate: 22
        },
        {
            name: "gold",
            amount: 33,
            lastUpdated: new Date(),
            perHourMiningRate: 33
        },
        {
            name: "water",
            amount: 44,
            lastUpdated: new Date(),
            perHourMiningRate: 44
        },
        {
            name: "sulfuricAcid",
            amount: 55,
            lastUpdated: new Date(),
            perHourMiningRate: 55
        },
        {
            name: "liquidNitrogen",
            amount: 66,
            lastUpdated: new Date(),
            perHourMiningRate: 66
        },
        {
            name: "hydrogen",
            amount: 77,
            lastUpdated: new Date(),
            perHourMiningRate: 77
        },
        {
            name: "oxygen",
            amount: 88,
            lastUpdated: new Date(),
            perHourMiningRate: 88
        },
        {
            name: "helium3",
            amount: 99,
            lastUpdated: new Date(),
            perHourMiningRate: 99
        }
    ];

    updatePlanetResources(planetId, resources);
}

export const getResourcesFromPlanet = async (planetId: ObjectId): Promise<Array<DBResource> | null> => {
    const planet = await getPlanetById(planetId);

    if(!planet) return null;

    return planet.resources;
}

export const updatePlanetResources = async (planetId: ObjectId, resources: Array<DBResource>) => {
    const planets = await Planets();
    await planets.updateOne({ _id: planetId }, {
        $set: {
            resources
        }
    });
}

export const calculateCurrentAvailableResources = async (planetId: ObjectId): Promise<Array<DBResource>> => {
    const resources = await getResourcesFromPlanet(planetId);

    if(resources === null) return [];

    resources.forEach(res => {
        const timeDiff = Math.abs((new Date()).getTime() - res.lastUpdated.getTime());
        const hours = timeDiff / (1000 * 60 * 60);
        const amountToAdd = hours * res.perHourMiningRate;
        res.amount += amountToAdd;
        res.lastUpdated = new Date();
    });


    updatePlanetResources(planetId, resources);
    return resources;
}