import { APIRoute } from "astro";
import { getPlanetById } from "../../../../lib/db/planets";
import { ObjectId } from "mongodb";
import Planet from "../../../../types/Planet";
import { calculateCurrentAvailableResources } from "../../../../lib/utils/resourceManager";
import validateAccessToken from "../../../../lib/utils/validateAccessToken";
import { getUserByAccessToken } from "../../../../lib/db/users";

export const GET: APIRoute = async ({ params, request }) => {
    const response = await validateAccessToken(request);
    if(response instanceof Response) return response;

    const user = await getUserByAccessToken(response);
    if(user === null) {
        return new Response(
            JSON.stringify({
                code: 401,
                message: "Unauthorized"
            }), { status: 401 }
        )
    }

    const planetId = params['planetId'];
    if(!planetId) return new Response(
        JSON.stringify({
            code: 400,
            message: "Bad Request"
        }), { status: 400 }
    );

    let planet: Planet | null;

    try {
        planet = await getPlanetById(new ObjectId(planetId));
        if(!planet) return new Response(
            JSON.stringify({
                code: 404,
                message: "Not Found"
            }), { status: 404 }
        );
    } catch(e) {
        return new Response(
            JSON.stringify({
                code: 500,
                message: "Internal Server Error"
            }), { status: 500 }
        );
    }

    if(planet.owner._id !== user._id) return new Response(
        JSON.stringify({
            code: 403,
            message: "Forbidden"
        }), { status: 403 }
    );

    await calculateCurrentAvailableResources(planet._id);

    return new Response(
        JSON.stringify({
            code: 200,
            message: "OK",
            data: JSON.parse(JSON.stringify(planet).replace(/_id/g, 'id'))
        })
    )
}